@@ -59,7 +59,7 @@ prepare_scene_list = function(scene, width = 400, height = 400, fov = 20,
                            "sphere" = 1,"xy_rect" = 2, "xz_rect" = 3,"yz_rect" = 4,"box" = 5, "triangle" = 6, 
                            "obj" = 7, "objcolor" = 8, "disk" = 9, "cylinder" = 10, "ellipsoid" = 11,
                            "objvertexcolor" = 12, "cone" = 13, "curve" = 14, "csg_object" = 15, "ply" = 16,
-                           "mesh3d" = 17))
+                           "mesh3d" = 17, "raymesh" = 18))
   typevec = unlist(lapply(tolower(scene$type),switch,
                           "diffuse" = 1,"metal" = 2,"dielectric" = 3, 
                           "oren-nayar" = 4, "light" = 5, "microfacet" = 6, 
@@ -325,11 +325,7 @@ prepare_scene_list = function(scene, width = 400, height = 400, fov = 20,
   if(is.null(focal_distance)) {
     focal_distance = sqrt(sum((lookfrom-lookat)^2))
   }
-  if(!is.null(options("cores")[[1]])) {
-    numbercores = options("cores")[[1]]
-  } else {
-    numbercores = parallel::detectCores()
-  }
+  numbercores = getOption("cores", default = getOption("Ncpus", default = parallel::detectCores()))
   if(!parallel) {
     numbercores = 1
   }
@@ -449,7 +445,7 @@ prepare_scene_list = function(scene, width = 400, height = 400, fov = 20,
   csg_info = list()
   csg_info$csg = csg_list
   
-  #mesh3d handler
+  #mesh3d/raymesh handler
   mesh_list = scene$mesh_info
   
   
